package net.ibizsys.central.plugin.boot.core.filter;

import com.alibaba.fastjson.annotation.JSONField;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.annotation.JsonAnySetter;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import net.ibizsys.central.dataentity.ds.IDEFilterDTORuntime;
import net.ibizsys.central.dataentity.service.IDEMethodDTORuntime;
import net.ibizsys.central.plugin.boot.core.domain.BaseData;
import net.ibizsys.central.plugin.boot.core.helper.FieldCache;
import net.ibizsys.central.plugin.boot.core.helper.QueryHelper;
import net.ibizsys.central.util.ISearchContextDTO;
import net.ibizsys.central.util.PageRequest;
import net.ibizsys.central.util.SearchContextDTO;
import net.ibizsys.model.dataentity.defield.IPSDEField;
import net.ibizsys.model.dataentity.ds.IPSDEDataSet;
import net.ibizsys.model.dataentity.ds.IPSDEFilterDTOField;
import net.ibizsys.runtime.util.*;
import org.springframework.cglib.beans.BeanMap;
import org.springframework.data.annotation.Transient;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.util.*;

@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
@Slf4j
public class BaseFilter<T> extends BaseData implements ISearchContextDTO {


    @JsonIgnore
    @JSONField(serialize = false)
    private transient IDEFilterDTORuntime iDEFilterDTORuntime = null;

    @JsonIgnore
    @JSONField(serialize = false)
    private IPSDEDataSet dataSet;

    @Override
    public void init(IDEMethodDTORuntime iDEMethodDTORuntime, Object objData, boolean bDTOData) throws Exception {

        if(!(iDEMethodDTORuntime instanceof IDEFilterDTORuntime)) {
            throw new Exception(String.format("传入实体方法DTO运行时对象不正确"));
        }

        if(!bDTOData) {
            throw new Exception(String.format("仅支持传入DTO"));
        }

        this.iDEFilterDTORuntime = (IDEFilterDTORuntime)iDEMethodDTORuntime;
        if (objData instanceof Map) {
            Map<String,Object> map = (Map<String,Object>)objData;

            for(java.util.Map.Entry<String, Object> entry : map.entrySet()) {
                if(entry.getKey().equalsIgnoreCase("searchconds")&&(!ObjectUtils.isEmpty(entry.getValue())))
                {
                    List list=(List)entry.getValue();
                    if(!(list.get(0) instanceof ISearchCond))
                        continue;
                }

                this.setObject(entry.getKey(), entry.getValue());
            }

        }
        onInit();
    }




    protected void onInit() throws Exception {
        Object objSearchConds = this.getSearchConds();
        if(objSearchConds != null) {
            //实际展开条件
            if(objSearchConds instanceof List) {
                List list = new ArrayList();
                ((List) objSearchConds).forEach(objItem->{
                    if(!(objItem instanceof Map)) {
                        return;
                    }
                    list.add(objItem);
                });
                for(Object objItem : list) {
                    ISearchCond iSearchCond = net.ibizsys.central.util.SearchGroupCond.getSearchCond((Map)objItem, true);
                    if(iSearchCond != null) {
                        this.getSearchCondsIf().add(iSearchCond);
                    }
                }
            }
        }
        if(this.getDEMethodDTORuntime()!=null) {
            //计算查询条件
            java.util.List<IPSDEFilterDTOField> psDEFilterDTOFieldList = this.getDEMethodDTORuntime().getPSDEFilterDTO().getPSDEFilterDTOFields();
            if(psDEFilterDTOFieldList != null) {
                for(IPSDEFilterDTOField iPSDEFilterDTOField : psDEFilterDTOFieldList) {
                    Object objValue = this.get(iPSDEFilterDTOField.getLowerCaseName());
                    if(ObjectUtils.isEmpty(objValue)) {
                        continue;
                    }

                    SearchFieldCond searchFieldCond = new SearchFieldCond();
                    searchFieldCond.setFieldName(iPSDEFilterDTOField.getPSDEFieldMust().getName());
                    searchFieldCond.setParamMode(true);
                    searchFieldCond.setCondOp(iPSDEFilterDTOField.getPSDEFSearchModeMust().getValueOP());
                    searchFieldCond.setValue(iPSDEFilterDTOField.getLowerCaseName());
                    searchFieldCond.setDataType(iPSDEFilterDTOField.getStdDataType());
                    searchFieldCond.setValueFunc(iPSDEFilterDTOField.getPSDEFSearchModeMust().getValueFunc());
                    if(StringUtils.isEmpty(searchFieldCond.getValueFunc())&&hasFieldCond(iPSDEFilterDTOField.getLowerCaseName()))
                        continue;
                    this.getSearchCondsIf().add(searchFieldCond);
                }
            }


            if(!ObjectUtils.isEmpty(query)) {
                //获取所有支持快速搜索的属性
                List<IPSDEField> psDEFieldList = this.getDEMethodDTORuntime().getDataEntityRuntime().getQuickSearchPSDEFields();
                if(psDEFieldList != null && psDEFieldList.size() > 0) {
                    SearchGroupCond searchGroupCond = new SearchGroupCond();
                    searchGroupCond.setCondOp(Conditions.OR);

                    for(IPSDEField iPSDEField: psDEFieldList ) {
                        SearchFieldCond searchFieldCond = new SearchFieldCond();
                        searchFieldCond.setFieldName(iPSDEField.getName());
                        searchFieldCond.setParamMode(true);
                        searchFieldCond.setCondOp(Conditions.LIKE);
                        searchFieldCond.setValue(PARAM_QUERY);
                        searchFieldCond.setDataType(iPSDEField.getStdDataType());

                        searchGroupCond.getSearchCondsIf().add(searchFieldCond);
                    }

                    this.getSearchCondsIf().add(searchGroupCond);
                }

            }

        }

        Object predefinedcond = this.get(PARAM_PREDEFINEDCOND);
        if(predefinedcond instanceof String) {
            if(!ObjectUtils.isEmpty(predefinedcond)) {
                SearchContextDTO.addSearchPredefinedCond(this, (String)predefinedcond, null);
            }
        }
    }


    @Override
    @JsonIgnore
    @JSONField(serialize = false)
    public IDEFilterDTORuntime getDEMethodDTORuntime() {
        return this.iDEFilterDTORuntime;
    }

    @Override
    public void setDEMethodDTORuntime(IDEFilterDTORuntime iDEFilterDTORuntime) {
        this.iDEFilterDTORuntime = iDEFilterDTORuntime;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    private List<ISearchCond> searchconds = null;
    @Override
    @JsonIgnore
    @JSONField(serialize = false)
    public List<ISearchCond> getSearchConds() {
        return searchconds;
    }
    @JsonIgnore
    @JSONField(serialize = false)
    public List<ISearchCond> getSearchCondsIf() {
        if(this.searchconds == null) {
            this.searchconds = new ArrayList<ISearchCond>();
        }
        return this.searchconds;
    }

    @Override
    public BaseFilter setFieldCond(String strFieldName, String strCondOp, Object objValue) {
        SearchFieldCond searchFieldCond = new SearchFieldCond();
        searchFieldCond.setFieldName(strFieldName);
        searchFieldCond.setCondOp(strCondOp);
        if(!Conditions.ISNULL.equals(strCondOp)
                && !Conditions.ISNOTNULL.equals(strCondOp)) {
            objValue="1";
        }
        searchFieldCond.setParamMode(true);

        String strParamName = String.format("n_%1$s_%2$s", strFieldName, strCondOp).toLowerCase();
        if(!hasFieldCond(strParamName)) {
            searchFieldCond.setValue(strParamName);
            this.getParamMap().put(strParamName, objValue);
            this.getSearchCondsIf().add(searchFieldCond);
        }

        return this;
    }

    public boolean hasFieldCond(String fieldCondName) {
        return this.getSearchCondsIf().stream().filter(f->f instanceof SearchFieldCond && fieldCondName.equals(((SearchFieldCond) f).getValue())).count()>0;
    }

    /**
     * 指定属性值等于条件值
     * @param strFieldName
     */
    @Override
    public BaseFilter eq(String strFieldName, Object objValue) {
        setFieldCond(strFieldName, Conditions.EQ, objValue);
        return this;
    }

    /**
     * 指定属性值不等于条件值
     * @param strFieldName
     */
    @Override
    public BaseFilter ne(String strFieldName, Object objValue) {
        setFieldCond(strFieldName, Conditions.NOTEQ, objValue);
        return this;
    }

    /**
     * 指定属性值大于条件值
     * @param strFieldName
     */
    @Override
    public BaseFilter gt(String strFieldName, Object objValue) {
        setFieldCond(strFieldName, Conditions.GT, objValue);
        return this;
    }

    /**
     * 指定属性值大于等于条件值
     * @param strFieldName
     */
    @Override
    public BaseFilter gte(String strFieldName, Object objValue) {
        setFieldCond(strFieldName, Conditions.GTANDEQ, objValue);
        return this;
    }

    /**
     * 指定属性值小于条件值
     * @param strFieldName
     */
    @Override
    public BaseFilter lt(String strFieldName, Object objValue) {
        setFieldCond(strFieldName, Conditions.LT, objValue);
        return this;
    }

    /**
     * 指定属性值小于等于条件值
     * @param strFieldName
     */
    @Override
    public BaseFilter lte(String strFieldName, Object objValue) {
        setFieldCond(strFieldName, Conditions.LTANDEQ, objValue);
        return this;
    }

    /**
     * 指定属性值匹配条件值
     * @param strFieldName
     */
    @Override
    public BaseFilter like(String strFieldName, Object objValue) {
        setFieldCond(strFieldName, Conditions.LIKE, objValue);
        return this;
    }

    /**
     * 指定属性值左侧匹配条件值
     * @param strFieldName
     */
    @Override
    public BaseFilter ll(String strFieldName, Object objValue) {
        setFieldCond(strFieldName, Conditions.LEFTLIKE, objValue);
        return this;
    }

    /**
     * 指定属性值右侧匹配条件值
     * @param strFieldName
     */
    @Override
    public BaseFilter rl(String strFieldName, Object objValue) {
        setFieldCond(strFieldName, Conditions.RIGHTLIKE, objValue);
        return this;
    }

    /**
     * 指定属性值为空
     * @param strFieldName
     */
    @Override
    public BaseFilter nvl(String strFieldName) {
        setFieldCond(strFieldName, Conditions.ISNULL, null);
        return this;
    }

    /**
     * 指定属性值不为空
     * @param strFieldName
     */
    @Override
    public BaseFilter nn(String strFieldName) {
        setFieldCond(strFieldName, Conditions.ISNOTNULL, null);
        return this;
    }

    /**
     * 指定属性值在值范围中
     * @param strFieldName
     */
    @Override
    public BaseFilter in(String strFieldName, Object objValue) {
        setFieldCond(strFieldName, Conditions.IN, objValue);
        return this;
    }

    /**
     * 指定属性值不在值范围中
     * @param strFieldName
     */
    @Override
    public BaseFilter ni(String strFieldName, Object objValue) {
        setFieldCond(strFieldName, Conditions.NOTIN, objValue);
        return this;
    }

    /**
     * 添加自定义条件
     * @param strCustomCond
     */
    @Override
    public BaseFilter custom(String strCustomCond) {
        SearchCustomCond searchCustomCond = new SearchCustomCond();
        searchCustomCond.setCustomCond(strCustomCond);
        this.getSearchCondsIf().add(searchCustomCond);
        return this;
    }

    /**
     * 添加预定义条件
     */
    @Override
    public BaseFilter predefined(String strPredefinedCond, String strPredefinedType) {
        SearchPredefinedCond searchPredefinedCond = new SearchPredefinedCond();
        searchPredefinedCond.setPredefinedCond(strPredefinedCond);
        searchPredefinedCond.setPredefinedType(strPredefinedType);
        this.getSearchCondsIf().add(searchPredefinedCond);
        return this;
    }



    @Override
    @JsonIgnore
    @JSONField(serialize = false)
    public net.ibizsys.central.util.SearchGroupCond and() {
        net.ibizsys.central.util.SearchGroupCond searchGroupCond = new net.ibizsys.central.util.SearchGroupCond(this);
        searchGroupCond.setCondOp(Conditions.AND);
        this.getSearchCondsIf().add(searchGroupCond);
        return searchGroupCond;
    }

    @Override
    @JsonIgnore
    @JSONField(serialize = false)
    public net.ibizsys.central.util.SearchGroupCond or() {
        net.ibizsys.central.util.SearchGroupCond searchGroupCond = new net.ibizsys.central.util.SearchGroupCond(this);
        searchGroupCond.setCondOp(Conditions.OR);
        this.getSearchCondsIf().add(searchGroupCond);
        return searchGroupCond;
    }





    @Override
    @JsonIgnore
    @JSONField(serialize = false)
    public BaseFilter all() {
        this.setCount(false);
        return this.limit(Integer.MAX_VALUE);
    }

    @Override
    public BaseFilter limit(int nSize) {
        return limit(0, nSize);
    }

    @Override
    public BaseFilter limit(int nStart, int nSize) {
        this.setPageable(STARTPAGE, nSize, nStart);
        return this;
    }


    /**
     * 设置直接变量值
     * @param strName
     * @param objValue
     * @return
     */
    @Override
    public BaseFilter param(String strName, Object objValue) {
        this.set(strName, objValue);
        return this;
    }


    @Override
    @JsonIgnore
    @JSONField(serialize = false)
    public String getCondOp() {
        return Conditions.AND;
    }

    @Override
    @JsonIgnore
    @JSONField(serialize = false)
    public boolean isNotMode() {
        return false;
    }

    @Override
    @JsonIgnore
    @JSONField(serialize = false)
    public String getCondType() {
        return CONDTYPE_GROUP;
    }


    public String query;

    public void setQuery() {

    }


    @Override
    @JsonIgnore
    @JSONField(serialize = false)
    public String getDrillDownCond() {
        Object drilldowncond = this.get(PARAM_DRILLDOWNCOND);
        if(drilldowncond instanceof String) {
            return (String)drilldowncond;
        }
        return null;
    }


    @Override
    @JsonIgnore
    @JSONField(serialize = false)
    public String getGroupCond() {
        Object groupcond = this.get(PARAM_GROUPCOND);
        if(groupcond instanceof String) {
            return (String)groupcond;
        }
        return null;
    }


    public void setCount(boolean count)
    {
        this.setObject("count",count);
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public boolean getCount(){return this.getBoolean("count",true).booleanValue();}

    @JsonIgnore
    @JSONField(serialize = false)
    public boolean isCount(){return this.getBoolean("count",true).booleanValue();}


    @Override
    public BaseFilter count(boolean bCount) {
        this.setObject("count",bCount);
        return this;
    }

    public Integer size = 20;
    public Integer page = 0;
    public Long offset = 0L;

    public String sort;
    public void setSort(String sort) {
        if (sort !=null) {
            sort = sort.trim();
            this.setPageSort(sort);
        }
        this.sort=sort;
    }
    @Override
    public BaseFilter sort(String strSortInfo) {
        this.setSort(strSortInfo);
        return this;
    }

    @JsonIgnore
    @JSONField(serialize = false)
    private Sort pageSort = null;
    public void setPageSort(String strSortInfo) {
        Sort pageSort = null;
        if (StringUtils.hasLength(strSortInfo)) {
            List<Sort.Order> list = new ArrayList<Sort.Order>();
            String[] parts = strSortInfo.split("[;]");
            for(String strPart : parts) {
                String[] items = strPart.split("[,]");
                if (items.length == 1) {
                    list.add(Sort.Order.asc(items[0]));
                } else if (items.length == 2) {
                    if ("DESC".equalsIgnoreCase(items[1])) {
                        list.add(Sort.Order.desc(items[0]));
                    } else {
                        list.add(Sort.Order.asc(items[0]));
                    }
                }
            }
            if (list.size() > 0) {
                pageSort = Sort.by(list);
            }
        }
        this.sort=strSortInfo;
        this.pageSort=pageSort;
    }
    @Override
    public void setPageSort(Sort pageSort) {
        String strSortInfo=null;
        if(pageSort != null && pageSort != Sort.unsorted()) {
            java.util.Iterator<Sort.Order> orders = pageSort.iterator();
            if(orders != null) {
                List<String> sortInfos = new ArrayList<>();
                while (orders.hasNext()) {
                    Sort.Order order = orders.next();
                    if(order.isAscending()) {
                        sortInfos.add(String.format("%1$s,asc", order.getProperty()));
                    }
                    else {
                        sortInfos.add(String.format("%1$s,desc", order.getProperty()));
                    }
                }
                if(sortInfos.size()>0)
                    strSortInfo=String.join(";",sortInfos);
            }
        }
        this.sort=strSortInfo;
        this.pageSort=pageSort;
    }


    @JsonIgnore
    @JSONField(serialize = false)
    private Pageable pageable = null;
    public void setPageable(Pageable pageable){
        this.page=pageable.getPageNumber();
        this.size=pageable.getPageSize();
        this.offset=pageable.getOffset();
        this.pageable=pageable;
    }
    public void setPageable(int nPageNumber, int nPageSize, long nOffset) {
        this.setPageable(PageRequest.of(nPageNumber, nPageSize, nOffset));
    }

    public Pageable getPageable() {
        if(pageable==null) {
            pageable=PageRequest.of(page, size, offset);
        }
        return pageable;
    }




    public Object get(String strName){
        return getObject(strName);
    }


    @JsonIgnore
    @JSONField(serialize = false)
    @Transient
    private BeanMap map;

    @JsonIgnore
    @JSONField(serialize = false)
    @Transient
    private BeanMap getMap()
    {
        if(map==null) {
            map=BeanMap.create(this);
        }
        return  map;
    }

    public Object getObject(String strName)
    {
        String fieldRealName= FieldCache.getFieldRealName(BaseFilter.class,strName);
        if(StringUtils.isEmpty(fieldRealName)&&(!this.getClass().isAssignableFrom(BaseFilter.class))) {
            fieldRealName=FieldCache.getFieldRealName(this.getClass(),strName);
        }
        if(!StringUtils.isEmpty(fieldRealName)) {
            return getMap().get(fieldRealName);
        }
        else {
            return this.getParamMap().get(strName);
        }
    }


    public QueryWrapper<T> getSearchCond(boolean gen)
    {
        return QueryHelper.getQueryWrapper(this);
    }

    public void modify(String field,Object val) {

    }

    public void set(String field, String value) {
        setObject(field,value);
    }

    public void set(String field, Object value) {
        setObject(field,value);
    }

    @JsonAnySetter
    @JSONField(name = "_any", unwrapped = true, serialize = false, deserialize = true)
    public void setObject(String field, Object value) {

        try
        {
            switch (field.toLowerCase())
            {
                case "size":
                    this.setSize(DataTypeUtils.getIntegerValue(value,20));
                    break;
                case "page":
                    this.setPage(DataTypeUtils.getIntegerValue(value,0));
                    break;
                case "offset":
                    this.setOffset(DataTypeUtils.getLongValue(value,0L));
                    break;
                case "searchconds":
                    if((!ObjectUtils.isEmpty(value))&&(value instanceof List))
                        this.setSearchconds((List)value);
                    break;
                case "query":
                    this.setQuery(DataTypeUtils.getStringValue(value,null));
                    break;
                case "pageable":
                    if(value instanceof Pageable)
                        this.setPageable((Pageable)value);
                case "pagesort":
                    if(value instanceof Sort)
                        this.setPageSort((Sort)value);
                    else
                        this.setPageSort(DataTypeUtils.getStringValue(value,null));
                    break;
                case "sort":
                    this.setSort(DataTypeUtils.getStringValue(value,null));
                    break;
                case "idefilterdtoruntime":
                    if(value instanceof IDEFilterDTORuntime)
                        this.setIDEFilterDTORuntime((IDEFilterDTORuntime)value);
                    break;
                default:
                    String fieldRealName=  FieldCache.getFieldRealName(this.getClass(),field);
                    if(!StringUtils.isEmpty(fieldRealName)) {
                        if (value == null) {
                            getMap().put(fieldRealName, null);
                        }
                        else {
                            getMap().put(fieldRealName, FieldCache.fieldValueOf(this.getClass(), fieldRealName, value));
                        }
                    }
                    else {
                        this.getParamMap().put(field,value);
                    }
            }
        }catch (Exception ex){}




    }


    @JsonIgnore
    @JSONField(serialize = false)
    public void resetAll() {
        getMap().keySet().forEach(key->{
            if((!key.toString().equalsIgnoreCase("dataSet"))&&(!key.toString().toLowerCase().endsWith("defilterdtoruntime")))
                map.put(key,null);
        });
        super.resetAll();
        page=0;
        size=20;
        offset=0L;
    }

}

