package net.ibizsys.central.plugin.boot.core.helper;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import net.ibizsys.central.dataentity.IDataEntityRuntime;
import net.ibizsys.central.dataentity.ds.IDEDataQueryCodeRuntime;
import net.ibizsys.central.plugin.boot.core.filter.BaseFilter;
import net.ibizsys.central.util.ISearchContext;
import net.ibizsys.model.dataentity.defield.IPSDEField;
import net.ibizsys.model.dataentity.ds.IPSDEDataQueryCodeExp;
import net.ibizsys.runtime.dataentity.defield.DEFTypes;
import net.ibizsys.runtime.security.UserContext;
import net.ibizsys.runtime.util.*;
import org.springframework.util.ObjectUtils;

import java.util.List;
import java.util.function.Consumer;

public class QueryHelper {

    public static <T> QueryWrapper<T> getQueryWrapper(BaseFilter filter)
    {
        QueryWrapper query=Wrappers.query();
        calcSearchCondItems(query,Conditions.AND,filter.getSearchConds());
        return query;
    }

    protected static <T> void calcSearchCondItems(QueryWrapper<T> query, String groupMode, List<ISearchCond> items) {
        int i = 0;
        for (ISearchCond iSearchCond : items) {
            if (i > 0) {
                if (groupMode.equals(Conditions.OR)) {
                    query.or();
                }
            }
            switch (iSearchCond.getCondType()) {
                case ISearchCond.CONDTYPE_GROUP:
                    searchCondGroup((ISearchGroupCond) iSearchCond,query);
                    break;
                case ISearchCond.CONDTYPE_DEFIELD:
                    searchDEField((ISearchFieldCond) iSearchCond,query);
                    break;
                case ISearchCond.CONDTYPE_CUSTOM:
                    query.apply(((ISearchCustomCond) iSearchCond).getCustomCond());
                    break;
            }
            i++;
        }
    }


    protected static <T> void searchCondGroup(ISearchGroupCond cond,QueryWrapper<T> query) {

        List<ISearchCond> items = cond.getSearchConds();
        if (!ObjectUtils.isEmpty(items)) {
            query.and(new Consumer<QueryWrapper<T>>() {
                @Override
                public void accept(QueryWrapper<T> objectQueryWrapper) {
                    switch (cond.getCondOp()) {
                        case Conditions.AND:
                            objectQueryWrapper.and(new Consumer<QueryWrapper<T>>() {
                                @Override
                                public void accept(QueryWrapper<T> subQuery) {
                                    calcSearchCondItems(subQuery, Conditions.AND, items);
                                }
                            });
                            break;
                        case Conditions.OR:
                            objectQueryWrapper.or(new Consumer<QueryWrapper<T>>() {
                                @Override
                                public void accept(QueryWrapper<T> subQuery) {
                                    calcSearchCondItems(subQuery, Conditions.OR, items);
                                }
                            });
                            break;
                    }
                }
            });


        }
    }

    protected static <T> void searchDEField(ISearchFieldCond cond,QueryWrapper<T> query) {

        String column = cond.getFieldName();

        Object val = cond.getValue();

        switch (cond.getCondOp()) {
            // 等于
            case Conditions.EQ:
                query.eq(column, val);
                break;
            // 不等于
            case Conditions.NOTEQ:
                query.ne(column, val);
                break;
            // 大于
            case Conditions.GT:
                query.gt(column, val);
                break;
            // 大于等于
            case Conditions.GTANDEQ:
                query.ge(column, val);
                break;
            // 小于
            case Conditions.LT:
                query.lt(column, val);
                break;
            // 小于等于
            case Conditions.LTANDEQ:
                query.le(column, val);
                break;
            // 为空
            case Conditions.ISNULL:
                query.isNull(column);
                break;
            // 不为空
            case Conditions.ISNOTNULL:
                query.isNotNull(column);
                break;
            // 文本包含
            case Conditions.LIKE:
            case Conditions.USERLIKE:
                query.like(column, val);
                break;
            // 值在范围中
            case Conditions.IN:
                query.in(column, val);
                break;
            // 值不在范围中
            case Conditions.NOTIN:
                query.notIn(column, val);
                break;
            // 文本左包含
            case Conditions.LEFTLIKE:
                query.likeRight(column, val);
                break;
            // 文本右包含
            case Conditions.RIGHTLIKE:
                query.likeLeft(column, val);
                break;
        }
    }
}
