package net.ibizsys.central.plugin.boot.core.runtime;

import com.alibaba.fastjson.JSON;
import net.ibizsys.central.ISystemGateway;
import net.ibizsys.central.dataentity.IDataEntityRuntimeContext;
import net.ibizsys.central.dataentity.service.DEServiceBase;
import net.ibizsys.central.dataentity.service.IDEService;
import net.ibizsys.central.plugin.boot.core.domain.BaseData;
import net.ibizsys.central.plugin.boot.core.filter.BaseFilter;
import net.ibizsys.central.plugin.boot.core.helper.StringAdvUtils;
import net.ibizsys.central.plugin.boot.core.service.IBaseService;
import net.ibizsys.central.plugin.boot.core.service.IMPService;
import net.ibizsys.central.plugin.boot.core.service.impl.BaseServiceImpl;
import net.ibizsys.central.plugin.boot.core.service.impl.MPServiceImpl;
import net.ibizsys.model.dataentity.action.IPSDEAction;
import net.ibizsys.model.dataentity.ds.IPSDEDataSet;
import net.ibizsys.model.dataentity.service.IPSDEMethodDTO;
import net.ibizsys.runtime.dataentity.service.DEMethodReturnTypes;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.util.Assert;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.util.HashMap;
import java.util.Map;

public class BootServiceImpl<T extends BaseData, F extends BaseFilter> extends DEServiceBase implements IBootService<T,F> {



    @Override
    public T createEntityDTO(IPSDEMethodDTO iPSDEMethodDTO) {
        if(iPSDEMethodDTO!=null&&iPSDEMethodDTO.getName()!=null&&iPSDEMethodDTO.getName().endsWith("DTO")) {
            try {
                T t = (T) this.getBaseService().getTClass().newInstance();
                return t;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    @Override
    public F createSearchContextDTO(IPSDEMethodDTO iPSDEMethodDTO) {
        try {
            F f = (F) this.getBaseService().getFClass().newInstance();
            return f;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    private IBootSystemRuntime systemRuntime;
    @Override
    public IBootSystemRuntime getSystemRuntime() {
        if(this.systemRuntime==null) {
            systemRuntime = BootSystemRuntime.getInstance();
        }
        return this.systemRuntime;
    }

    public void setSystemRuntime(IBootSystemRuntime systemRuntime) {
        this.systemRuntime=systemRuntime;
    }

    private IDataEntityRuntimeContext dataEntityRuntimeContext;

    public IDataEntityRuntimeContext getDataEntityRuntimeContext() {
        return dataEntityRuntimeContext;
    }

    public void setDataEntityRuntimeContext(IDataEntityRuntimeContext dataEntityRuntimeContext) {
        if(this.dataEntityRuntimeContext==null)
            this.dataEntityRuntimeContext = dataEntityRuntimeContext;
    }

    private String dataEntityId;
    public void setDataEntityId(String dataEntityId)
    {
        this.dataEntityId = dataEntityId;
    }
    @Override
    public String getDataEntityId() {
        return dataEntityId;
    }

    private IBaseService baseService;
    public void setBaseService(IBaseService baseService)
    {
        if(baseService==null)
            return;
        this.baseService = baseService;
        this.setDataEntityId(baseService.getDataEntityId());

    }
    public IBaseService getBaseService(){
        return baseService;
    }

    public BootServiceImpl(){}
    public BootServiceImpl(IBaseService baseService) {
        this.setBaseService(baseService);
        BootSystemRuntime.addLocalDEServiceMap(baseService.getDataEntityId(),this);
    }


    public Object executeAction(String strActionName, IPSDEAction iPSDEAction, Object[] args) throws Throwable {
        if(getBaseService()!=null) {
            if(args.length>0&&args[0]!=null) {
                Method method = getBaseService().getServiceMethod(StringAdvUtils.camelcase(strActionName), args[0].getClass());
                if (method != null) {
                    Object rt = null;
                    if (iPSDEAction != null && iPSDEAction.getPSDEActionReturn() != null && DEMethodReturnTypes.VOID.equals(iPSDEAction.getPSDEActionReturn().getType()))
                        try{
                            method.invoke(this.getBaseService(), args[0]);
                        } catch (InvocationTargetException ex){
                            throw ex.getTargetException();
                        }

                    else {
                        try {
                            rt = method.invoke(this.getBaseService(), args[0]);
                        } catch (InvocationTargetException ex){
                            throw ex.getTargetException();
                        }
                    }
                    return rt;
                }
            }
            return getBaseService().executeAction(strActionName, iPSDEAction, args);
        }
        return this.getDataEntityRuntime().executeAction(strActionName, iPSDEAction, args, true);
    }

    public Object executeActionReal(IDataEntityRuntimeContext iDataEntityRuntimeContext, String strActionName, IPSDEAction iPSDEAction, Object[] args, Object actionData) throws Throwable{
        Assert.notNull(iDataEntityRuntimeContext, "传入实体运行时上下文对象无效");
        this.setDataEntityRuntimeContext(iDataEntityRuntimeContext);
        if(getBaseService()!=null) {
            if(args.length>0&&args[0]!=null) {
                Method method = getBaseService().getServiceMethod("on"+StringAdvUtils.pascalcase(strActionName), args[0].getClass());
                if (method != null) {
                    Object rt = null;
                    if (iPSDEAction != null && iPSDEAction.getPSDEActionReturn() != null && DEMethodReturnTypes.VOID.equals(iPSDEAction.getPSDEActionReturn().getType()))
                    {
                        try {
                            method.invoke(this.getBaseService(), args[0]);
                        } catch (InvocationTargetException ex){
                            throw ex.getTargetException();
                        }
                    }
                    else {
                        try {
                            rt = method.invoke(this.getBaseService(), args[0]);
                        }catch (InvocationTargetException ex){
                            throw ex.getTargetException();
                        }
                    }

                    return rt;
                }
            }
            return getBaseService().executeActionReal(iDataEntityRuntimeContext, strActionName, iPSDEAction, args, actionData);
        }
        return iDataEntityRuntimeContext.executeActionReal(strActionName, iPSDEAction, args, actionData);
    }



    public Object fetchDataSet(String strDataSetName, IPSDEDataSet iPSDEDataSet, Object[] args) throws Throwable {
        if(getBaseService()!=null) {
            if(args.length>0&&args[0]!=null) {
                ((F) args[0]).setDataSet(iPSDEDataSet);
                Method method = getBaseService().getServiceMethod("fetch" + StringAdvUtils.pascalcase(strDataSetName), args[0].getClass());
                if (method != null) {
                    try {
                        return method.invoke(this.getBaseService(), args[0]);
                    }catch (InvocationTargetException ex){
                        throw ex.getTargetException();
                    }
                }
            }
            return getBaseService().fetchDataSet(strDataSetName,iPSDEDataSet,args);
        }
        return this.getDataEntityRuntime().fetchDataSet(strDataSetName, iPSDEDataSet, args, true);
    }

    public Object fetchDataSetReal(IDataEntityRuntimeContext iDataEntityRuntimeContext, String strDataSetName, IPSDEDataSet iPSDEDataSet, Object[] args, Object actionData) throws Throwable{
        Assert.notNull(iDataEntityRuntimeContext, "传入实体运行时上下文对象无效");
        this.setDataEntityRuntimeContext(iDataEntityRuntimeContext);
        if(getBaseService()!=null) {
            if(args.length>0&&args[0]!=null) {
                ((F) args[0]).setDataSet(iPSDEDataSet);
                Method method = getBaseService().getServiceMethod("onFetch" + StringAdvUtils.pascalcase(strDataSetName), args[0].getClass());
                if (method != null) {
                    try {
                        return method.invoke(this.getBaseService(), args[0]);
                    }catch (InvocationTargetException ex){
                        throw ex.getTargetException();
                    }
                }
            }
            return getBaseService().fetchDataSetReal(iDataEntityRuntimeContext, strDataSetName, iPSDEDataSet, args, actionData);
        }
        return iDataEntityRuntimeContext.fetchDataSetReal(strDataSetName, iPSDEDataSet, args, actionData);
    }


}

