package net.ibizsys.central.plugin.boot.core.service;

import net.ibizsys.central.dataentity.IDataEntityRuntime;
import net.ibizsys.central.dataentity.IDataEntityRuntimeContext;
import net.ibizsys.central.plugin.boot.core.domain.BaseData;
import net.ibizsys.central.plugin.boot.core.filter.BaseFilter;
import net.ibizsys.central.plugin.boot.core.helper.StringAdvUtils;
import net.ibizsys.central.plugin.boot.core.runtime.IBootService;
import net.ibizsys.central.plugin.boot.core.runtime.IBootSystemRuntime;
import net.ibizsys.model.dataentity.action.IPSDEAction;
import net.ibizsys.model.dataentity.ds.IPSDEDataSet;
import org.springframework.data.domain.Page;
import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

public interface IBaseService<T extends BaseData, F extends BaseFilter> {

    String getDataEntityId();

    default IBootService getRuntimeService() {
        return null;
    }

    default void setRuntimeService(IBootService runtimeService) { }

    default IBootSystemRuntime getSystemRuntime(){
        return getRuntimeService().getSystemRuntime();
    }

    default IDataEntityRuntime getDataEntityRuntime(){
        return getRuntimeService().getDataEntityRuntime();
    }

    Method getServiceMethod(String name, Class clazz);

    Class<T> getTClass();

    Class<F> getFClass();

    default IDataEntityRuntimeContext getDataEntityRuntimeContext() {
        return getRuntimeService().getDataEntityRuntimeContext();
    }

    default Object executeAction(String strActionName, IPSDEAction iPSDEAction, Object[] args) throws Throwable {
        return this.getRuntimeService().getDataEntityRuntime().executeAction(strActionName, iPSDEAction, args, true);
    }

    default void callAction(String strActionName,Object arg)  throws Throwable{
        beforeAction(strActionName,arg);
        forwardAction(strActionName,arg);
        afterAction(strActionName,arg);
    }

    default void beforeAction(String strActionName,Object arg)  throws Throwable{
        try {
            Method beforeMethod = getServiceMethod("before" + StringAdvUtils.pascalcase(strActionName), arg.getClass());
            if (beforeMethod != null)
                beforeMethod.invoke(this, arg);
        }catch (InvocationTargetException ex){
            throw ex.getTargetException();
        }
    }

    default Object forwardAction(String strActionName,Object arg)  throws Throwable{
        return this.getRuntimeService().getDataEntityRuntime().executeAction(strActionName, null, new Object[]{arg}, true);
    }

    default Object executeActionReal(IDataEntityRuntimeContext iDataEntityRuntimeContext, String strActionName, IPSDEAction iPSDEAction, Object[] args, Object actionData) throws Throwable{
        return iDataEntityRuntimeContext.executeActionReal(strActionName, iPSDEAction, args, actionData);
    }

    default Object onRealAction(String strActionName,Object arg)  throws Throwable{
        return this.getDataEntityRuntimeContext().executeActionReal(strActionName, null, new Object[]{arg}, null);
    }

    default void afterAction(String strActionName,Object arg)  throws Throwable{
        try {
            Method afterMethod = getServiceMethod("after" + StringAdvUtils.pascalcase(strActionName), arg.getClass());
            if (afterMethod != null)
                afterMethod.invoke(this, arg);
        } catch (InvocationTargetException ex){
            throw ex.getTargetException();
        }
    }

    default Object fetchDataSet(String strDataSetName, IPSDEDataSet iPSDEDataSet, Object[] args) throws Throwable {
        return this.getRuntimeService().getDataEntityRuntime().fetchDataSet(strDataSetName, iPSDEDataSet, args, true);
    }

    default void beforeFetch(String strActionName,Object arg)  throws Throwable{
        try {
            Method beforeMethod = getServiceMethod("beforeFetch" + StringAdvUtils.pascalcase(strActionName), arg.getClass());
            if (beforeMethod != null)
                beforeMethod.invoke(this, arg);
        } catch (InvocationTargetException ex){
            throw ex.getTargetException();
        }
    }

    default Object forwardFetch(String dsName,F arg) throws Throwable{
        return this.getRuntimeService().getDataEntityRuntime().fetchDataSet(dsName, arg.getDataSet(), new Object[]{arg}, true);
    }

    default Object fetchDataSetReal(IDataEntityRuntimeContext iDataEntityRuntimeContext, String strDataSetName, IPSDEDataSet iPSDEDataSet, Object[] args, Object actionData) throws Throwable{
        return iDataEntityRuntimeContext.fetchDataSetReal(strDataSetName, iPSDEDataSet, args, actionData);
    }

    default Object onRealFetch(String dsName,F arg) throws Throwable{
        return this.getDataEntityRuntimeContext().fetchDataSetReal("DEFAULT", arg.getDataSet(), new Object[]{arg}, null);
    }

    default boolean create(T dto) throws Throwable {
        callAction("Create",dto);
        return true;
    }

    default boolean onCreate(T dto) throws Throwable {
        onRealAction("Create",dto);
        return true;
    }

    default boolean update(T dto) throws Throwable {
        callAction("Update",dto);
        return true;
    }

    default boolean onUpdate(T dto) throws Throwable {
        onRealAction("Update",dto);
        return true;
    }

    default boolean save(T dto) throws Throwable {
        callAction("Save",dto);
        return true;
    }

    default boolean onSave(T dto) throws Throwable {
        onRealAction("Save",dto);
        return true;
    }

    default <K extends Serializable> boolean remove(K key) throws Throwable {
        beforeAction("Remove",key);
        List keys=new ArrayList();
        keys.add(key);
        if(!remove(keys))
            return false;
        afterAction("Remove",key);
        return true;
    }

    default boolean remove(List keys) throws Throwable {
        callAction("Remove",keys);
        return true;
    }

    default boolean onRemove(List keys) throws Throwable {
        onRealAction("Remove",keys);
        return true;
    }

    default <K extends Serializable> T get(K key) throws Throwable {
        T dto = (T)forwardAction("Get",key);
        if(dto!=null)
            afterAction("Get",dto);
        return dto;
    }

    default <K extends Serializable> T onGet(K key) throws Throwable {
        return (T)onRealAction("Get",key);
    }

    default T getDraft(T dto) throws Throwable {
        beforeAction("GetDraft",dto);
        T rtdto = (T)forwardAction("GetDraft",dto);
        if(dto!=null)
            afterAction("GetDraft",rtdto);
        return rtdto;
    }

    default T onGetDraft(T dto) throws Throwable {
        return (T)onRealAction("GetDraft",dto);
    }

    default Integer checkKey(T dto) throws Throwable {
        beforeAction("CheckKey",dto);
        return  (Integer) forwardAction("CheckKey",dto);
    }

    default Integer onCheckKey(T dto) throws Throwable {
        return (Integer) onRealAction("CheckKey",dto);
    }



    default Page<T> fetchDefault(F dto) throws Throwable {
        beforeFetch("Default",dto);
        return (Page) forwardFetch("DEFAULT",dto);
    }

    default Page<T> onFetchDefault(F dto) throws Throwable {
        return (Page) onRealFetch("DEFAULT",dto);
    }

    default List<T> selectDefault(F dto) throws Throwable {
        return (List)this.getRuntimeService().getDataEntityRuntime().selectDataQuery("DEFAULT", dto);
    }

    default List<T> selectSimple(F dto) throws Throwable {
        return (List)this.getRuntimeService().getDataEntityRuntime().selectDataQuery("SIMPLE", dto);
    }

    default List<T> selectView(F dto) throws Throwable {
        return (List)this.getRuntimeService().getDataEntityRuntime().selectDataQuery("VIEW", dto);
    }

}
