package net.ibizsys.central.plugin.boot.core.service.impl;

import net.ibizsys.central.plugin.boot.core.domain.BaseData;
import net.ibizsys.central.plugin.boot.core.filter.BaseFilter;
import net.ibizsys.central.plugin.boot.core.runtime.BootServiceImpl;
import net.ibizsys.central.plugin.boot.core.runtime.IBootService;
import net.ibizsys.central.plugin.boot.core.service.IBaseService;
import net.ibizsys.central.plugin.boot.core.service.IMPService;
import net.ibizsys.central.plugin.boot.core.service.IServiceApiService;
import org.apache.commons.lang3.ObjectUtils;

import javax.annotation.PostConstruct;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.util.HashMap;
import java.util.Map;

public  abstract class BaseServiceImpl<T extends BaseData, F extends BaseFilter> implements IBaseService<T, F> {


    private IBootService runtimeService;
    @Override
    public IBootService getRuntimeService(){
        return runtimeService;
    }

    public void setRuntimeService(IBootService runtimeService) {
        this.runtimeService = runtimeService;
    }

    @PostConstruct
    private void postConstruct() throws Exception {
        Class serviceClass=this.getClass();
        while (serviceClass!=null) {
            if(!ObjectUtils.isEmpty(serviceClass.getInterfaces())) {
                Class interfaceClass=serviceClass.getInterfaces()[0];
                if(!ObjectUtils.isEmpty(interfaceClass.getInterfaces())) {
                    if(interfaceClass.getInterfaces()[0].isAssignableFrom(IBaseService.class)
                            ||interfaceClass.getInterfaces()[0].isAssignableFrom(IMPService.class)
                            ||interfaceClass.getInterfaces()[0].isAssignableFrom(IServiceApiService.class)) {
                        serviceClass=interfaceClass;
                        break;
                    }
                }
            }
            serviceClass = serviceClass.getSuperclass();
        }
        if(!ObjectUtils.isEmpty(serviceClass.getGenericInterfaces())) {
            ParameterizedType ptype = (ParameterizedType) serviceClass.getGenericInterfaces()[0];
            tClass = (Class) ptype.getActualTypeArguments()[0];
            fClass = (Class) ptype.getActualTypeArguments()[1];
        }
        runtimeService = new BootServiceImpl<T,F>(this);
    }


    private Class<T> tClass;
    private Class<F> fClass;

    public Class<T> getTClass() {
        return tClass;
    }

    public Class<F> getFClass() {
        return fClass;
    }

    private Object lockMethods = new Object();

    private Map<String, Method> serviceMethods=new HashMap<>();

    public Method getServiceMethod(String name,Class clazz) {
        Method method = null;
        synchronized (lockMethods)
        {
            String tag=name;
            if(clazz.isArray())
                tag=tag+"List";
            if(serviceMethods.containsKey(tag))
                return serviceMethods.get(tag);

            try {
                method=this.getClass().getMethod(name,clazz);

            } catch (NoSuchMethodException e) {
            }
            serviceMethods.put(tag,method);

        }
        return method;
    }

}

